@@ -43,6 +43,7 @@
 #include "xsltutils.h"
 #include "imports.h"
 #include "documents.h"
+#include "security.h"
 
 
 
@@ -70,14 +71,14 @@ xsltParseStylesheetImport(xsltStylesheetPtr style, xmlNodePtr cur) {
     xmlChar *uriRef = NULL;
     xmlChar *URI = NULL;
     xsltStylesheetPtr res;
+    xsltSecurityPrefsPtr sec;
 
     if ((cur == NULL) || (style == NULL))
 	return (ret);
 
     uriRef = xsltGetNsProp(cur, (const xmlChar *)"href", XSLT_NAMESPACE);
     if (uriRef == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:import : missing href attribute\n");
 	goto error;
     }
@@ -85,15 +86,30 @@ xsltParseStylesheetImport(xsltStylesheetPtr style, xmlNodePtr cur) {
     base = xmlNodeGetBase(style->doc, cur);
     URI = xmlBuildURI(uriRef, base);
     if (URI == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:import : invalid URI reference %s\n", uriRef);
 	goto error;
     }
+
+    /*
+     * Security framework check
+     */
+    sec = xsltGetDefaultSecurityPrefs();
+    if (sec != NULL) {
+	int secres;
+
+	secres = xsltCheckRead(sec, NULL, URI);
+	if (secres == 0) {
+	    xsltTransformError(NULL, NULL, NULL,
+		 "xsl:import: read rights for %s denied\n",
+			     URI);
+	    goto error;
+	}
+    }
+
     import = xmlParseFile((const char *)URI);
     if (import == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:import : unable to load %s\n", URI);
 	goto error;
     }
@@ -144,8 +160,7 @@ xsltParseStylesheetInclude(xsltStylesheetPtr style, xmlNodePtr cur) {
 
     uriRef = xsltGetNsProp(cur, (const xmlChar *)"href", XSLT_NAMESPACE);
     if (uriRef == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:include : missing href attribute\n");
 	goto error;
     }
@@ -153,16 +168,14 @@ xsltParseStylesheetInclude(xsltStylesheetPtr style, xmlNodePtr cur) {
     base = xmlNodeGetBase(style->doc, cur);
     URI = xmlBuildURI(uriRef, base);
     if (URI == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:include : invalid URI reference %s\n", uriRef);
 	goto error;
     }
 
     include = xsltLoadStyleDocument(style, URI);
     if (include == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:include : unable to load %s\n", URI);
 	goto error;
     }
