@@ -9,6 +9,7 @@
  * daniel@veillard.com
  */
 
+#define IN_LIBXSLT
 #include "libxslt.h"
 
 #include <string.h>
@@ -69,16 +70,14 @@ xsltNamespaceAlias(xsltStylesheetPtr style, xmlNodePtr node) {
     sprefix = xsltGetNsProp(node, (const xmlChar *)"stylesheet-prefix",
 	                   XSLT_NAMESPACE);
     if (sprefix == NULL) {
-	xsltPrintErrorContext(NULL, style, node);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, node,
 	    "namespace-alias: stylesheet-prefix attribute missing\n");
 	return;
     }
     rprefix = xsltGetNsProp(node, (const xmlChar *)"result-prefix",
 	                   XSLT_NAMESPACE);
     if (rprefix == NULL) {
-	xsltPrintErrorContext(NULL, style, node);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, node,
 	    "namespace-alias: result-prefix attribute missing\n");
 	goto error;
     }
@@ -88,8 +87,7 @@ xsltNamespaceAlias(xsltStylesheetPtr style, xmlNodePtr node) {
 	sNs = xmlSearchNs(node->doc, node, sprefix);
     }
     if ((sNs == NULL) || (sNs->href == NULL)) {
-	xsltPrintErrorContext(NULL, style, node);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, node,
 	    "namespace-alias: prefix %s not bound to any namespace\n",
 	                 sprefix);
 	goto error;
@@ -100,8 +98,7 @@ xsltNamespaceAlias(xsltStylesheetPtr style, xmlNodePtr node) {
 	rNs = xmlSearchNs(node->doc, node, rprefix);
     }
     if ((rNs == NULL) || (rNs->href == NULL)) {
-	xsltPrintErrorContext(NULL, style, node);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, node,
 	    "namespace-alias: prefix %s not bound to any namespace\n",
 	                 rprefix);
 	goto error;
@@ -109,8 +106,7 @@ xsltNamespaceAlias(xsltStylesheetPtr style, xmlNodePtr node) {
     if (style->nsAliases == NULL)
 	style->nsAliases = xmlHashCreate(10);
     if (style->nsAliases == NULL) {
-	xsltPrintErrorContext(NULL, style, node);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, node,
 	    "namespace-alias: cannot create hash table\n");
 	goto error;
     }
@@ -196,7 +192,7 @@ xsltGetNamespace(xsltTransformContextPtr ctxt, xmlNodePtr cur, xmlNsPtr ns,
     while (style != NULL) {
 	if (style->nsAliases != NULL)
 	    URI = (const xmlChar *) 
-		xmlHashLookup(ctxt->style->nsAliases, ns->href);
+		xmlHashLookup(style->nsAliases, ns->href);
 	if (URI != NULL)
 	    break;
 
@@ -236,7 +232,7 @@ xsltGetNamespace(xsltTransformContextPtr ctxt, xmlNodePtr cur, xmlNsPtr ns,
 xmlNsPtr
 xsltCopyNamespaceList(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	              xmlNsPtr cur) {
-    xmlNsPtr ret = NULL;
+    xmlNsPtr ret = NULL, tmp;
     xmlNsPtr p = NULL,q;
     const xmlChar *URI;
 
@@ -254,6 +250,22 @@ xsltCopyNamespaceList(xsltTransformContextPtr ctxt, xmlNodePtr node,
     while (cur != NULL) {
 	if (cur->type != XML_NAMESPACE_DECL)
 	    break;
+
+	/*
+	 * Avoid duplicating namespace declrations on the tree
+	 */
+	if ((node != NULL) && (node->ns != NULL) &&
+            (xmlStrEqual(node->ns->href, cur->href)) &&
+            (xmlStrEqual(node->ns->prefix, cur->prefix))) {
+	    cur = cur->next;
+	    continue;
+	}
+	tmp = xmlSearchNs(node->doc, node, cur->prefix);
+	if ((tmp != NULL) && (xmlStrEqual(tmp->href, cur->href))) {
+	    cur = cur->next;
+	    continue;
+	}
+	
 	if (!xmlStrEqual(cur->href, XSLT_NAMESPACE)) {
 	    /* TODO apply cascading */
 	    URI = (const xmlChar *) xmlHashLookup(ctxt->style->nsAliases,
