@@ -8,6 +8,7 @@
 
 #include "libxslt/libxslt.h"
 #include "libexslt/exslt.h"
+#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -43,12 +44,14 @@
 #include <libxml/catalog.h>
 #endif
 #include <libxml/parserInternals.h>
+#include <libxml/uri.h>
 
 #include <libxslt/xslt.h>
 #include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 #include <libxslt/extensions.h>
+#include <libxslt/security.h>
 
 #include <libexslt/exsltconfig.h>
 
@@ -88,6 +91,7 @@ xmlParserInputPtr xmlNoNetExternalEntityLoader(const char *URL,
 static int debug = 0;
 static int repeat = 0;
 static int timing = 0;
+static int dumpextensions = 0;
 static int novalid = 0;
 static int noout = 0;
 #ifdef LIBXML_DOCB_ENABLED
@@ -102,14 +106,96 @@ static int xinclude = 0;
 static int profile = 0;
 
 #define MAX_PARAMETERS 64
+#define MAX_PATHS 64
 
 static const char *params[MAX_PARAMETERS + 1];
 static int nbparams = 0;
 static xmlChar *strparams[MAX_PARAMETERS + 1];
 static int nbstrparams = 0;
+static xmlChar *paths[MAX_PATHS + 1];
+static int nbpaths = 0;
 static const char *output = NULL;
 static int errorno = 0;
+static const char *writesubtree = NULL;
 
+/*
+ * Entity loading control and customization.
+ */
+static
+void parsePath(const xmlChar *path) {
+    const xmlChar *cur;
+
+    if (path == NULL)
+	return;
+    while (*path != 0) {
+	if (nbpaths >= MAX_PATHS) {
+	    fprintf(stderr, "MAX_PATHS reached: too many paths\n");
+	    return;
+	}
+	cur = path;
+	while ((*cur == ' ') || (*cur == ':'))
+	    cur++;
+	path = cur;
+	while ((*cur != 0) && (*cur != ' ') && (*cur != ':'))
+	    cur++;
+	if (cur != path) {
+	    paths[nbpaths] = xmlStrndup(path, cur - path);
+	    if (paths[nbpaths] != NULL)
+		nbpaths++;
+	    path = cur;
+	}
+    }
+}
+
+xmlExternalEntityLoader defaultEntityLoader = NULL;
+
+static xmlParserInputPtr 
+xsltprocExternalEntityLoader(const char *URL, const char *ID,
+			     xmlParserCtxtPtr ctxt) {
+    xmlParserInputPtr ret;
+    warningSAXFunc warning = NULL;
+
+    int i;
+
+    if ((ctxt != NULL) && (ctxt->sax != NULL)) {
+	warning = ctxt->sax->warning;
+	ctxt->sax->warning = NULL;
+    }
+
+    if (defaultEntityLoader != NULL) {
+	ret = defaultEntityLoader(URL, ID, ctxt);
+	if (ret != NULL) {
+	    if (warning != NULL)
+		ctxt->sax->warning = warning;
+	    return(ret);
+	}
+    }
+    for (i = 0;i < nbpaths;i++) {
+	xmlChar *newURL;
+	int len;
+
+	len = xmlStrlen(paths[i]) + xmlStrlen(BAD_CAST URL) + 5;
+	newURL = xmlMalloc(len);
+	if (newURL != NULL) {
+	    snprintf(newURL, len, "%s/%s", paths[i], URL);
+	    ret = defaultEntityLoader((const char *)newURL, ID, ctxt);
+	    xmlFree(newURL);
+	    if (ret != NULL) {
+		if (warning != NULL)
+		    ctxt->sax->warning = warning;
+		return(ret);
+	    }
+	}
+    }
+    if (warning != NULL) {
+	ctxt->sax->warning = warning;
+	if (URL != NULL)
+	    warning(ctxt, "failed to load external entity \"%s\"\n", URL);
+	else if (ID != NULL)
+	    warning(ctxt, "failed to load external entity \"%s\"\n", ID);
+    }
+    return(NULL);
+}
 
 /*
  * Internal timing routines to remove the necessity to have unix-specific
@@ -232,10 +318,34 @@ static void endTimer(char *format, ...)
 }
 #endif
 
+/*
+ * xsltSubtreeCheck:
+ *
+ * allow writes only on a subtree specified on the command line
+ */
+static int
+xsltSubtreeCheck(xsltSecurityPrefsPtr sec ATTRIBUTE_UNUSED,
+	          xsltTransformContextPtr ctxt ATTRIBUTE_UNUSED,
+		  const char *value ATTRIBUTE_UNUSED) {
+    int len, ret;
+
+    if (writesubtree == NULL)
+	return(0);
+    if (value == NULL)
+	return(-1);
+
+    len = xmlStrlen(BAD_CAST writesubtree);
+    ret = xmlStrncmp(BAD_CAST writesubtree, BAD_CAST value, len);
+    if (ret == 0)
+	return(1);
+    return(0);
+}
+
 static void
 xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
     xmlDocPtr res;
     xsltTransformContextPtr ctxt;
+   
 
 #ifdef LIBXML_XINCLUDE_ENABLED
     if (xinclude) {
@@ -282,6 +392,8 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 	}
 	if (ctxt->state == XSLT_STATE_ERROR)
 	    errorno = 9;
+	if (ctxt->state == XSLT_STATE_STOPPED)
+	    errorno = 10;
 	xsltFreeTransformContext(ctxt);
 	if (timing) {
 	    if (repeat)
@@ -331,7 +443,21 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 
 	xmlFreeDoc(res);
     } else {
-	xsltRunStylesheet(cur, doc, params, output, NULL, NULL);
+	int ret;
+
+	ctxt = xsltNewTransformContext(cur, doc);
+	if (ctxt == NULL)
+	    return;
+	if (profile) {
+	    ret = xsltRunStylesheetUser(cur, doc, params, output,
+		                        NULL, NULL, stderr, ctxt);
+	} else {
+	    ret = xsltRunStylesheetUser(cur, doc, params, output,
+		                        NULL, NULL, NULL, ctxt);
+	}
+	if (ctxt->state == XSLT_STATE_ERROR)
+	    errorno = 9;
+	xsltFreeTransformContext(ctxt);
 	if (timing)
 	    endTimer("Running stylesheet and saving result");
 	xmlFreeDoc(doc);
@@ -347,6 +473,7 @@ static void usage(const char *name) {
     printf("\t--timing: display the time used\n");
     printf("\t--repeat: run the transformation 20 times\n");
     printf("\t--debug: dump the tree of the result instead\n");
+    printf("\t--dumpextensions: dump the registered extension elements and functions to stdout\n");
     printf("\t--novalid skip the Dtd loading phase\n");
     printf("\t--noout: do not dump the result\n");
     printf("\t--maxdepth val : increase the maximum depth\n");
@@ -357,11 +484,15 @@ static void usage(const char *name) {
     printf("\t--docbook: the input document is SGML docbook\n");
 #endif
     printf("\t--param name value : pass a (parameter,value) pair\n");
-    printf("\t       value is an XPath expression.\n");
+    printf("\t       value is an UTF8 XPath expression.\n");
     printf("\t       string values must be quoted like \"'string'\"\n or");
     printf("\t       use stringparam to avoid it\n");
-    printf("\t--stringparam name value : pass a (parameter,string value) pair\n");
-    printf("\t--nonet refuse to fetch DTDs or entities over network\n");
+    printf("\t--stringparam name value : pass a (parameter, UTF8 string value) pair\n");
+    printf("\t--path 'paths': provide a set of paths for resources\n");
+    printf("\t--nonet : refuse to fetch DTDs or entities over network\n");
+    printf("\t--nowrite : refuse to write to any file or resource\n");
+    printf("\t--nomkdir : refuse to create directories\n");
+    printf("\t--writesubtree path : allow file write only with the path subtree\n");
 #ifdef LIBXML_CATALOG_ENABLED
     printf("\t--catalogs : use SGML catalogs from $SGML_CATALOG_FILES\n");
     printf("\t             otherwise XML Catalogs starting from \n");
@@ -381,6 +512,7 @@ main(int argc, char **argv)
     int i;
     xsltStylesheetPtr cur = NULL;
     xmlDocPtr doc, style;
+    xsltSecurityPrefsPtr sec = NULL;
 
     if (argc <= 1) {
         usage(argv[0]);
@@ -392,6 +524,10 @@ main(int argc, char **argv)
     LIBXML_TEST_VERSION
 
     xmlLineNumbersDefault(1);
+    sec = xsltNewSecurityPrefs();
+    xsltSetDefaultSecurityPrefs(sec);
+    defaultEntityLoader = xmlGetExternalEntityLoader();
+    xmlSetExternalEntityLoader(xsltprocExternalEntityLoader);
 
     for (i = 1; i < argc; i++) {
         if (!strcmp(argv[i], "-"))
@@ -458,7 +594,29 @@ main(int argc, char **argv)
             profile++;
         } else if ((!strcmp(argv[i], "-nonet")) ||
                    (!strcmp(argv[i], "--nonet"))) {
-            xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
+	    defaultEntityLoader = xmlNoNetExternalEntityLoader;
+        } else if ((!strcmp(argv[i], "-nowrite")) ||
+                   (!strcmp(argv[i], "--nowrite"))) {
+	    xsltSetSecurityPrefs(sec, XSLT_SECPREF_WRITE_FILE,
+		                 xsltSecurityForbid);
+	    xsltSetSecurityPrefs(sec, XSLT_SECPREF_CREATE_DIRECTORY,
+		                 xsltSecurityForbid);
+	    xsltSetSecurityPrefs(sec, XSLT_SECPREF_WRITE_NETWORK,
+		                 xsltSecurityForbid);
+        } else if ((!strcmp(argv[i], "-nomkdir")) ||
+                   (!strcmp(argv[i], "--nomkdir"))) {
+	    xsltSetSecurityPrefs(sec, XSLT_SECPREF_CREATE_DIRECTORY,
+		                 xsltSecurityForbid);
+        } else if ((!strcmp(argv[i], "-writesubtree")) ||
+                   (!strcmp(argv[i], "--writesubtree"))) {
+	    i++;
+	    writesubtree = argv[i];
+	    xsltSetSecurityPrefs(sec, XSLT_SECPREF_WRITE_FILE,
+		                 xsltSubtreeCheck);
+        } else if ((!strcmp(argv[i], "-path")) ||
+                   (!strcmp(argv[i], "--path"))) {
+	    i++;
+	    parsePath(BAD_CAST argv[i]);
 #ifdef LIBXML_CATALOG_ENABLED
         } else if ((!strcmp(argv[i], "-catalogs")) ||
                    (!strcmp(argv[i], "--catalogs"))) {
@@ -526,7 +684,11 @@ main(int argc, char **argv)
                 if (value > 0)
                     xsltMaxDepth = value;
             }
-        } else {
+        } else if ((!strcmp(argv[i],"-dumpextensions"))||
+			(!strcmp(argv[i],"--dumpextensions"))) {
+		dumpextensions++;
+		
+	} else {
             fprintf(stderr, "Unknown option %s\n", argv[i]);
             usage(argv[0]);
             return (3);
@@ -551,6 +713,9 @@ main(int argc, char **argv)
     exsltRegisterAll();
     xsltRegisterTestModule();
 
+    if (dumpextensions) 
+	xsltDebugDumpExtensions(NULL);
+
     for (i = 1; i < argc; i++) {
         if ((!strcmp(argv[i], "-maxdepth")) ||
             (!strcmp(argv[i], "--maxdepth"))) {
@@ -561,6 +726,14 @@ main(int argc, char **argv)
                    (!strcmp(argv[i], "--output"))) {
             i++;
 	    continue;
+        } else if ((!strcmp(argv[i], "-writesubtree")) ||
+                   (!strcmp(argv[i], "--writesubtree"))) {
+            i++;
+	    continue;
+        } else if ((!strcmp(argv[i], "-path")) ||
+                   (!strcmp(argv[i], "--path"))) {
+            i++;
+	    continue;
 	}
         if ((!strcmp(argv[i], "-param")) || (!strcmp(argv[i], "--param"))) {
             i += 2;
@@ -591,6 +764,10 @@ main(int argc, char **argv)
 		}
 		cur = xsltParseStylesheetDoc(style);
 		if (cur != NULL) {
+		    if (cur->errors != 0) {
+			errorno = 5;
+			goto done;
+		    }
 		    if (cur->indent == 1)
 			xmlIndentTreeOutput = 1;
 		    else
@@ -649,6 +826,7 @@ done:
 #if 0
     xmlMemoryDump();
 #endif
+    xsltFreeSecurityPrefs(sec);
     return(errorno);
 }
 
